﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using foxit.pdf;
using PDFReader_Demo_CSharp.Common;

namespace PDFReader_Demo_CSharp.UI
{
    public enum SearchContextState
    {
        SearchNone,
        SearchStart,
        SearchNoFind,
        SearchFind,
        SearchFindEnd,
        SearchStop
    }

    public class SearchEventArgs : EventArgs
    {
        internal SearchEventArgs(SearchContextState state, int page_index, foxit.pdf.TextSearch text_search)
        {
            State = state;
            PageIndex = page_index;
            SearchHandle = text_search;
        }

        public SearchContextState State { get; set; }
        public int PageIndex { get; set; }
        public TextSearch SearchHandle { get; set; }
    }

    class SearchContext : IDisposable
    {
        private PDFDoc pdf_doc_;
        private bool is_searching_ = false;
        private int start_find_page_index_;
        private int current_page_index_;
        private string search_string_ = null;
        private int flag_;
        private foxit.pdf.TextSearch text_search_;
        private bool has_find_;

        public event EventHandler<SearchEventArgs> SearchResult;

        public SearchContext(PDFDoc pdf_doc)
        {
            this.pdf_doc_ = pdf_doc;
        }

        ~SearchContext()
        {
            Dispose();
        }

        public void Dispose()
        {
            pdf_doc_ = null;
        }

        public bool IsSearching
        {
            get
            {
                return this.is_searching_;
            }
        }

        public void StartSearch(int page_index, string string_to_find, bool is_match_case_sensitive, bool is_match_whole_word, int direction)
        {
            if (string_to_find.Equals(""))
            {
                System.Windows.Forms.MessageBox.Show("Searching content is blank!", "Information", System.Windows.Forms.MessageBoxButtons.OK);
                return;
            }

            start_find_page_index_ = page_index;
            current_page_index_ = start_find_page_index_;
            search_string_ = string_to_find;
            is_searching_ = true;
            has_find_ = false;

            if (is_match_case_sensitive) { flag_ |= (int)TextSearch.SearchFlags.e_SearchMatchCase; }
            if (is_match_whole_word) { flag_ |= (int)TextSearch.SearchFlags.e_SearchMatchWholeWord; }

            if (direction > 0)
                direction = -1;

            SearchNext(direction == 0 ? true : false);
        }

        public void StopSearch()
        {
            is_searching_ = false;
            search_string_ = null;
            flag_ = 0;
            start_find_page_index_ = -1;
            text_search_ = null;

            OnSearchResult(SearchContextState.SearchStop,  -1, null);
        }

        public void SearchNext(bool is_forward = true)
        {
            bool is_match = false, is_end = false;
            int page_count = pdf_doc_.GetPageCount();

            while (!is_match)
            {
                PDFPage current_page = null;
                if (text_search_ == null)
                {
                    current_page = FoxitUtil.LoadPage(pdf_doc_, current_page_index_);
                    if (current_page == null)
                        return;

                    text_search_ = new TextSearch(new TextPage(current_page, (int)TextPage.TextParseFlags.e_ParseTextNormal));
                    text_search_.SetSearchFlags((int)flag_);
                    text_search_.SetStartPage(current_page_index_);
                    text_search_.SetEndPage(current_page_index_);
                    text_search_.SetPattern(search_string_);
                }

                if (is_forward)
                    is_match = text_search_.FindNext();
                else
                    is_match = text_search_.FindPrev();

                if (!is_match)
                {
                    text_search_ = null;
                    if (is_forward)
                        current_page_index_++;
                    else
                        current_page_index_--;

                    if (current_page_index_ >= page_count || current_page_index_ < 0)
                    {
                        if (current_page_index_ >= page_count) current_page_index_ = page_count - 1;
                        if (current_page_index_ < 0) current_page_index_ = 0;
                        is_end = true;
                        if (current_page != null) current_page.Dispose();
                        break;
                    }

                    if (current_page_index_ == start_find_page_index_)
                    {
                        if (!has_find_)
                        {
                            if (current_page != null) current_page.Dispose();
                            break;
                        }
                    }
                }
                else
                {
                    if (!has_find_)
                        has_find_ = true;
                }

                if (current_page != null) current_page.Dispose();
            }
            

            OnSearchResult(is_end ? SearchContextState.SearchFindEnd : is_match ? SearchContextState.SearchFind : SearchContextState.SearchNoFind, is_match ? current_page_index_ : -1, text_search_);
        }

        protected void OnSearchResult(SearchContextState state, int page_index, foxit.pdf.TextSearch text_search)
        {
            SearchEventArgs search_event_args = new SearchEventArgs(state, page_index, text_search);

            OnSearchResult(search_event_args);
        }

        protected void OnSearchResult(SearchEventArgs e)
        {
            EventHandler<SearchEventArgs> handler = SearchResult;
            if (handler != null)
            {
                handler(this, e);
            }
        }
    }
}
